﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SQLite;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using WSCF;

namespace RemoteSQLite
{
    public class SQLiteRemote : ISQLiteRemoteService
    {
        NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();
        public DateTime Now { get; private set; }

        public event PropertyChangedEventHandler PropertyChanged;

        System.Timers.Timer timer;
        public SQLiteRemote()
        {
            timer = new System.Timers.Timer();
            timer.AutoReset = true;
            timer.Interval = 1000;
            timer.Elapsed += Timer_Elapsed;
            timer.Start();
        }

        private void Timer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            Now = DateTime.Now;
        }

        public void SetTime(DateTime dt)
        {
            //转换System.DateTime到SYSTEMTIME

            SYSTEMTIME st = new SYSTEMTIME();

            st.FromDateTime(dt);
            

            //调用Win32 API设置系统时间

            Win32API.SetLocalTime(ref st);
        }

        /// <summary> 
        /// 查询数据库中的所有数据类型信息。
        /// </summary> 
        public void ExecuteReader(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<DataTable> task = new Task<DataTable>(() =>
            {
                DataTable data;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        // 开始读取
                        SQLiteDataAdapter adapter = new SQLiteDataAdapter(command);
                        data = new DataTable();
                        adapter.Fill(data);
                        connection.Close();
                    }
                }
                return data;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error($"ExecuteReader{connectionString}, {commandText} task.IsFaulted==true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e= new TimeoutException();
                logger.Error(e, $"ExecuteReader {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteReaderReponse()
                {
                    ErrMsg = e.ToString(),
                });
                throw e;
            }
        }

        public void ExecuteNonQuery(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<int> task = new Task<int>(() =>
            {
                int ret = 0;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        ret = command.ExecuteNonQuery();
                        connection.Close();
                    }
                }
                return ret;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error($"ExecuteNonQuery {connectionString}, {commandText} ttask.IsFaulted==true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e = new TimeoutException();
                logger.Error(e,$"ExecuteNonQuery {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteNonQueryReponse()
                {
                    ErrMsg = e.Message
                });
            }
        }

        public void ExecuteScalar(string connectionString, string commandText, int millisecondsTimeout, AsyncCBHandler asyncDelegate, object asyncContext)
        {
            Task<object> task = new Task<object>(() =>
            {
                object ret;
                using (SQLiteConnection connection = new SQLiteConnection(connectionString))
                {
                    using (SQLiteCommand command = new SQLiteCommand(connection))
                    {
                        connection.Open();
                        command.CommandText = commandText;
                        ret = command.ExecuteScalar();
                        connection.Close();
                    }
                }
                return ret;
            });
            task.Start();
            task.Wait(millisecondsTimeout);

            if (task.IsFaulted)
            {
                logger.Error(task.Exception, $"ExecuteScalar {connectionString}, {commandText} task.IsFaulted=true");
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    ErrMsg = task.Exception.ToString(),
                });
            }
            if (task.IsCompleted)
            {
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    Ret = task.Result
                });
            }
            else
            {
                Exception e = new TimeoutException();
                logger.Error(e, $"ExecuteScalar {connectionString}, {commandText}");
                asyncDelegate?.Invoke(asyncContext, new ExecuteScalarReponse()
                {
                    ErrMsg = e.Message
                });
            }
        }

        
    }

    /// <summary>

    /// 

    /// </summary>

    public struct SYSTEMTIME

    {

        public ushort wYear;

        public ushort wMonth;

        public ushort wDayOfWeek;

        public ushort wDay;

        public ushort wHour;

        public ushort wMinute;

        public ushort wSecond;

        public ushort wMilliseconds;



        /// <summary>

        /// 从System.DateTime转换。

        /// </summary>

        /// <param name="time">System.DateTime类型的时间。</param>

        public void FromDateTime(DateTime time)

        {

            wYear = (ushort)time.Year;

            wMonth = (ushort)time.Month;

            wDayOfWeek = (ushort)time.DayOfWeek;

            wDay = (ushort)time.Day;

            wHour = (ushort)time.Hour;

            wMinute = (ushort)time.Minute;

            wSecond = (ushort)time.Second;

            wMilliseconds = (ushort)time.Millisecond;

        }

        /// <summary>

        /// 转换为System.DateTime类型。

        /// </summary>

        /// <returns></returns>

        public DateTime ToDateTime()

        {

            return new DateTime(wYear, wMonth, wDay, wHour, wMinute, wSecond, wMilliseconds);

        }

        /// <summary>

        /// 静态方法。转换为System.DateTime类型。

        /// </summary>

        /// <param name="time">SYSTEMTIME类型的时间。</param>

        /// <returns></returns>

        public static DateTime ToDateTime(SYSTEMTIME time)

        {

            return time.ToDateTime();

        }

    }

    public class Win32API

    {

        [DllImport("Kernel32.dll")]

        public static extern bool SetLocalTime(ref SYSTEMTIME Time);

        [DllImport("Kernel32.dll")]

        public static extern void GetLocalTime(ref SYSTEMTIME Time);

    }


}
